<?php

namespace Drupal\media_entity_download\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Drupal\media_entity\MediaInterface;
use Symfony\Component\Filesystem\Exception\FileNotFoundException;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;

/**
 * Class DownloadController.
 */
class DownloadController extends ControllerBase {

  /**
   * Serves the file upon request.
   *
   * @param MediaInterface $media
   *   A valid media object.
   *
   * @return BinaryFileResponse
   *   Serve the file as the response.
   *
   * @throws \Exception
   * @throws FileNotFoundException
   */
  public function download(MediaInterface $media) {
    $plugin_id = $media->getType()->getPluginId();
    $config = $this->config('media_entity.bundle.' . $plugin_id);
    $type_config = $config->get('type_configuration');
    $field = $type_config['source_field'];

    if (!$field) {
      throw new \Exception("No source field configured for the {$plugin_id} media type.");
    }

    $fid = $media->{$field}->target_id;

    // if media has no file item
    if (!$fid) {
      throw new \Exception("The media item requested has no file referenced/uploaded in the {$field} field.");
    }

    $file = $this->entityTypeManager()->getStorage('file')->load($fid);

    // or file entity could not be loaded
    if (!$file) {
      throw new \Exception("File id {$fid} could not be loaded.");
    }

    $uri = $file->getFileUri();
    $filename = $file->getFilename();

    // or item does not exist on disk
    if (!file_exists($uri)) {
      throw new FileNotFoundException(NULL, 0, NULL, $uri);
    }

    $response = new BinaryFileResponse($uri);
    $response->setContentDisposition(
      ResponseHeaderBag::DISPOSITION_ATTACHMENT,
      $filename
    );

    return $response;
  }

}
